@@ -1,5 +1,6 @@
 """Constants for well-known URIs used throughout the A2A Python SDK."""
 
-AGENT_CARD_WELL_KNOWN_PATH = '/.well-known/agent.json'
+AGENT_CARD_WELL_KNOWN_PATH = '/.well-known/agent-card.json'
+PREV_AGENT_CARD_WELL_KNOWN_PATH = '/.well-known/agent.json'
 EXTENDED_AGENT_CARD_PATH = '/agent/authenticatedExtendedCard'
 DEFAULT_RPC_URL = '/'
