@@ -2,6 +2,7 @@
 """Utils for converting between proto and Python types."""
 
 import json
+import logging
 import re
 
 from typing import Any
@@ -13,9 +14,36 @@ from a2a.grpc import a2a_pb2
 from a2a.utils.errors import ServerError
 
 
+logger = logging.getLogger(__name__)
+
+
 # Regexp patterns for matching
-_TASK_NAME_MATCH = r'tasks/(\w+)'
-_TASK_PUSH_CONFIG_NAME_MATCH = r'tasks/(\w+)/pushNotificationConfigs/(\w+)'
+_TASK_NAME_MATCH = re.compile(r'tasks/([\w-]+)')
+_TASK_PUSH_CONFIG_NAME_MATCH = re.compile(
+    r'tasks/([\w-]+)/pushNotificationConfigs/([\w-]+)'
+)
+
+
+def dict_to_struct(dictionary: dict[str, Any]) -> struct_pb2.Struct:
+    """Converts a Python dict to a Struct proto.
+
+    Unfortunately, using `json_format.ParseDict` does not work because this
+    wants the dictionary to be an exact match of the Struct proto with fields
+    and keys and values, not the traditional Python dict structure.
+
+    Args:
+      dictionary: The Python dict to convert.
+
+    Returns:
+      The Struct proto.
+    """
+    struct = struct_pb2.Struct()
+    for key, val in dictionary.items():
+        if isinstance(val, dict):
+            struct[key] = dict_to_struct(val)
+        else:
+            struct[key] = val
+    return struct
 
 
 class ToProto:
@@ -26,12 +54,12 @@ class ToProto:
         if message is None:
             return None
         return a2a_pb2.Message(
-            message_id=message.messageId,
-            content=[ToProto.part(p) for p in message.parts],
-            context_id=message.contextId,
-            task_id=message.taskId,
+            message_id=message.message_id,
+            content=[cls.part(p) for p in message.parts],
+            context_id=message.context_id or '',
+            task_id=message.task_id or '',
             role=cls.role(message.role),
-            metadata=ToProto.metadata(message.metadata),
+            metadata=cls.metadata(message.metadata),
         )
 
     @classmethod
@@ -40,56 +68,49 @@ class ToProto:
     ) -> struct_pb2.Struct | None:
         if metadata is None:
             return None
-        return struct_pb2.Struct(
-            # TODO: Add support for other types.
-            fields={
-                key: struct_pb2.Value(string_value=value)
-                for key, value in metadata.items()
-                if isinstance(value, str)
-            }
-        )
+        return dict_to_struct(metadata)
 
     @classmethod
     def part(cls, part: types.Part) -> a2a_pb2.Part:
         if isinstance(part.root, types.TextPart):
             return a2a_pb2.Part(text=part.root.text)
         if isinstance(part.root, types.FilePart):
-            return a2a_pb2.Part(file=ToProto.file(part.root.file))
+            return a2a_pb2.Part(file=cls.file(part.root.file))
         if isinstance(part.root, types.DataPart):
-            return a2a_pb2.Part(data=ToProto.data(part.root.data))
+            return a2a_pb2.Part(data=cls.data(part.root.data))
         raise ValueError(f'Unsupported part type: {part.root}')
 
     @classmethod
     def data(cls, data: dict[str, Any]) -> a2a_pb2.DataPart:
-        json_data = json.dumps(data)
-        return a2a_pb2.DataPart(
-            data=json_format.Parse(
-                json_data,
-                struct_pb2.Struct(),
-            )
-        )
+        return a2a_pb2.DataPart(data=dict_to_struct(data))
 
     @classmethod
     def file(
         cls, file: types.FileWithUri | types.FileWithBytes
     ) -> a2a_pb2.FilePart:
         if isinstance(file, types.FileWithUri):
-            return a2a_pb2.FilePart(file_with_uri=file.uri)
-        return a2a_pb2.FilePart(file_with_bytes=file.bytes.encode('utf-8'))
+            return a2a_pb2.FilePart(
+                file_with_uri=file.uri, mime_type=file.mime_type, name=file.name
+            )
+        return a2a_pb2.FilePart(
+            file_with_bytes=file.bytes.encode('utf-8'),
+            mime_type=file.mime_type,
+            name=file.name,
+        )
 
     @classmethod
     def task(cls, task: types.Task) -> a2a_pb2.Task:
         return a2a_pb2.Task(
             id=task.id,
-            context_id=task.contextId,
-            status=ToProto.task_status(task.status),
+            context_id=task.context_id,
+            status=cls.task_status(task.status),
             artifacts=(
-                [ToProto.artifact(a) for a in task.artifacts]
+                [cls.artifact(a) for a in task.artifacts]
                 if task.artifacts
                 else None
             ),
             history=(
-                [ToProto.message(h) for h in task.history]  # type: ignore[misc]
+                [cls.message(h) for h in task.history]  # type: ignore[misc]
                 if task.history
                 else None
             ),
@@ -98,8 +119,8 @@ class ToProto:
     @classmethod
     def task_status(cls, status: types.TaskStatus) -> a2a_pb2.TaskStatus:
         return a2a_pb2.TaskStatus(
-            state=ToProto.task_state(status.state),
-            update=ToProto.message(status.message),
+            state=cls.task_state(status.state),
+            update=cls.message(status.message),
         )
 
     @classmethod
@@ -117,17 +138,19 @@ class ToProto:
                 return a2a_pb2.TaskState.TASK_STATE_FAILED
             case types.TaskState.input_required:
                 return a2a_pb2.TaskState.TASK_STATE_INPUT_REQUIRED
+            case types.TaskState.auth_required:
+                return a2a_pb2.TaskState.TASK_STATE_AUTH_REQUIRED
             case _:
                 return a2a_pb2.TaskState.TASK_STATE_UNSPECIFIED
 
     @classmethod
     def artifact(cls, artifact: types.Artifact) -> a2a_pb2.Artifact:
         return a2a_pb2.Artifact(
-            artifact_id=artifact.artifactId,
+            artifact_id=artifact.artifact_id,
             description=artifact.description,
-            metadata=ToProto.metadata(artifact.metadata),
+            metadata=cls.metadata(artifact.metadata),
             name=artifact.name,
-            parts=[ToProto.part(p) for p in artifact.parts],
+            parts=[cls.part(p) for p in artifact.parts],
         )
 
     @classmethod
@@ -144,7 +167,7 @@ class ToProto:
         cls, config: types.PushNotificationConfig
     ) -> a2a_pb2.PushNotificationConfig:
         auth_info = (
-            ToProto.authentication_info(config.authentication)
+            cls.authentication_info(config.authentication)
             if config.authentication
             else None
         )
@@ -160,12 +183,12 @@ class ToProto:
         cls, event: types.TaskArtifactUpdateEvent
     ) -> a2a_pb2.TaskArtifactUpdateEvent:
         return a2a_pb2.TaskArtifactUpdateEvent(
-            task_id=event.taskId,
-            context_id=event.contextId,
-            artifact=ToProto.artifact(event.artifact),
-            metadata=ToProto.metadata(event.metadata),
+            task_id=event.task_id,
+            context_id=event.context_id,
+            artifact=cls.artifact(event.artifact),
+            metadata=cls.metadata(event.metadata),
             append=event.append or False,
-            last_chunk=event.lastChunk or False,
+            last_chunk=event.last_chunk or False,
         )
 
     @classmethod
@@ -173,10 +196,10 @@ class ToProto:
         cls, event: types.TaskStatusUpdateEvent
     ) -> a2a_pb2.TaskStatusUpdateEvent:
         return a2a_pb2.TaskStatusUpdateEvent(
-            task_id=event.taskId,
-            context_id=event.contextId,
-            status=ToProto.task_status(event.status),
-            metadata=ToProto.metadata(event.metadata),
+            task_id=event.task_id,
+            context_id=event.context_id,
+            status=cls.task_status(event.status),
+            metadata=cls.metadata(event.metadata),
             final=event.final,
         )
 
@@ -187,13 +210,13 @@ class ToProto:
         if not config:
             return a2a_pb2.SendMessageConfiguration()
         return a2a_pb2.SendMessageConfiguration(
-            accepted_output_modes=list(config.acceptedOutputModes),
-            push_notification=ToProto.push_notification_config(
-                config.pushNotificationConfig
+            accepted_output_modes=config.accepted_output_modes,
+            push_notification=cls.push_notification_config(
+                config.push_notification_config
             )
-            if config.pushNotificationConfig
+            if config.push_notification_config
             else None,
-            history_length=config.historyLength,
+            history_length=config.history_length,
             blocking=config.blocking or False,
         )
 
@@ -206,19 +229,7 @@ class ToProto:
         | types.TaskArtifactUpdateEvent,
     ) -> a2a_pb2.StreamResponse:
         """Converts a task, message, or task update event to a StreamResponse."""
-        if isinstance(event, types.TaskStatusUpdateEvent):
-            return a2a_pb2.StreamResponse(
-                status_update=ToProto.task_status_update_event(event)
-            )
-        if isinstance(event, types.TaskArtifactUpdateEvent):
-            return a2a_pb2.StreamResponse(
-                artifact_update=ToProto.task_artifact_update_event(event)
-            )
-        if isinstance(event, types.Message):
-            return a2a_pb2.StreamResponse(msg=ToProto.message(event))
-        if isinstance(event, types.Task):
-            return a2a_pb2.StreamResponse(task=ToProto.task(event))
-        raise ValueError(f'Unsupported event type: {type(event)}')
+        return cls.stream_response(event)
 
     @classmethod
     def task_or_message(
@@ -250,18 +261,20 @@ class ToProto:
             return a2a_pb2.StreamResponse(
                 status_update=cls.task_status_update_event(event),
             )
-        return a2a_pb2.StreamResponse(
-            artifact_update=cls.task_artifact_update_event(event),
-        )
+        if isinstance(event, types.TaskArtifactUpdateEvent):
+            return a2a_pb2.StreamResponse(
+                artifact_update=cls.task_artifact_update_event(event),
+            )
+        raise ValueError(f'Unsupported event type: {type(event)}')
 
     @classmethod
     def task_push_notification_config(
         cls, config: types.TaskPushNotificationConfig
     ) -> a2a_pb2.TaskPushNotificationConfig:
         return a2a_pb2.TaskPushNotificationConfig(
-            name=f'tasks/{config.taskId}/pushNotificationConfigs/{config.taskId}',
+            name=f'tasks/{config.task_id}/pushNotificationConfigs/{config.push_notification_config.id}',
             push_notification_config=cls.push_notification_config(
-                config.pushNotificationConfig,
+                config.push_notification_config,
             ),
         )
 
@@ -272,20 +285,37 @@ class ToProto:
     ) -> a2a_pb2.AgentCard:
         return a2a_pb2.AgentCard(
             capabilities=cls.capabilities(card.capabilities),
-            default_input_modes=list(card.defaultInputModes),
-            default_output_modes=list(card.defaultOutputModes),
+            default_input_modes=list(card.default_input_modes),
+            default_output_modes=list(card.default_output_modes),
             description=card.description,
-            documentation_url=card.documentationUrl,
+            documentation_url=card.documentation_url,
             name=card.name,
             provider=cls.provider(card.provider),
             security=cls.security(card.security),
-            security_schemes=cls.security_schemes(card.securitySchemes),
+            security_schemes=cls.security_schemes(card.security_schemes),
             skills=[cls.skill(x) for x in card.skills] if card.skills else [],
             url=card.url,
             version=card.version,
             supports_authenticated_extended_card=bool(
-                card.supportsAuthenticatedExtendedCard
+                card.supports_authenticated_extended_card
             ),
+            preferred_transport=card.preferred_transport,
+            protocol_version=card.protocol_version,
+            additional_interfaces=[
+                cls.agent_interface(x) for x in card.additional_interfaces
+            ]
+            if card.additional_interfaces
+            else None,
+        )
+
+    @classmethod
+    def agent_interface(
+        cls,
+        interface: types.AgentInterface,
+    ) -> a2a_pb2.AgentInterface:
+        return a2a_pb2.AgentInterface(
+            transport=interface.transport,
+            url=interface.url,
         )
 
     @classmethod
@@ -294,7 +324,24 @@ class ToProto:
     ) -> a2a_pb2.AgentCapabilities:
         return a2a_pb2.AgentCapabilities(
             streaming=bool(capabilities.streaming),
-            push_notifications=bool(capabilities.pushNotifications),
+            push_notifications=bool(capabilities.push_notifications),
+            extensions=[
+                cls.extension(x) for x in capabilities.extensions or []
+            ],
+        )
+
+    @classmethod
+    def extension(
+        cls,
+        extension: types.AgentExtension,
+    ) -> a2a_pb2.AgentExtension:
+        return a2a_pb2.AgentExtension(
+            uri=extension.uri,
+            description=extension.description,
+            params=dict_to_struct(extension.params)
+            if extension.params
+            else None,
+            required=extension.required,
         )
 
     @classmethod
@@ -315,16 +362,12 @@ class ToProto:
     ) -> list[a2a_pb2.Security] | None:
         if not security:
             return None
-        rval: list[a2a_pb2.Security] = []
-        for s in security:
-            rval.append(
-                a2a_pb2.Security(
-                    schemes={
-                        k: a2a_pb2.StringList(list=v) for (k, v) in s.items()
-                    }
-                )
+        return [
+            a2a_pb2.Security(
+                schemes={k: a2a_pb2.StringList(list=v) for (k, v) in s.items()}
             )
-        return rval
+            for s in security
+        ]
 
     @classmethod
     def security_schemes(
@@ -353,7 +396,7 @@ class ToProto:
                 http_auth_security_scheme=a2a_pb2.HTTPAuthSecurityScheme(
                     description=scheme.root.description,
                     scheme=scheme.root.scheme,
-                    bearer_format=scheme.root.bearerFormat,
+                    bearer_format=scheme.root.bearer_format,
                 )
             )
         if isinstance(scheme.root, types.OAuth2SecurityScheme):
@@ -363,46 +406,52 @@ class ToProto:
                     flows=cls.oauth2_flows(scheme.root.flows),
                 )
             )
+        if isinstance(scheme.root, types.MutualTLSSecurityScheme):
+            return a2a_pb2.SecurityScheme(
+                mtls_security_scheme=a2a_pb2.MutualTlsSecurityScheme(
+                    description=scheme.root.description,
+                )
+            )
         return a2a_pb2.SecurityScheme(
             open_id_connect_security_scheme=a2a_pb2.OpenIdConnectSecurityScheme(
                 description=scheme.root.description,
-                open_id_connect_url=scheme.root.openIdConnectUrl,
+                open_id_connect_url=scheme.root.open_id_connect_url,
             )
         )
 
     @classmethod
     def oauth2_flows(cls, flows: types.OAuthFlows) -> a2a_pb2.OAuthFlows:
-        if flows.authorizationCode:
+        if flows.authorization_code:
             return a2a_pb2.OAuthFlows(
                 authorization_code=a2a_pb2.AuthorizationCodeOAuthFlow(
-                    authorization_url=flows.authorizationCode.authorizationUrl,
-                    refresh_url=flows.authorizationCode.refreshUrl,
-                    scopes=dict(flows.authorizationCode.scopes.items()),
-                    token_url=flows.authorizationCode.tokenUrl,
+                    authorization_url=flows.authorization_code.authorization_url,
+                    refresh_url=flows.authorization_code.refresh_url,
+                    scopes=dict(flows.authorization_code.scopes.items()),
+                    token_url=flows.authorization_code.token_url,
                 ),
             )
-        if flows.clientCredentials:
+        if flows.client_credentials:
             return a2a_pb2.OAuthFlows(
                 client_credentials=a2a_pb2.ClientCredentialsOAuthFlow(
-                    refresh_url=flows.clientCredentials.refreshUrl,
-                    scopes=dict(flows.clientCredentials.scopes.items()),
-                    token_url=flows.clientCredentials.tokenUrl,
+                    refresh_url=flows.client_credentials.refresh_url,
+                    scopes=dict(flows.client_credentials.scopes.items()),
+                    token_url=flows.client_credentials.token_url,
                 ),
             )
         if flows.implicit:
             return a2a_pb2.OAuthFlows(
                 implicit=a2a_pb2.ImplicitOAuthFlow(
-                    authorization_url=flows.implicit.authorizationUrl,
-                    refresh_url=flows.implicit.refreshUrl,
+                    authorization_url=flows.implicit.authorization_url,
+                    refresh_url=flows.implicit.refresh_url,
                     scopes=dict(flows.implicit.scopes.items()),
                 ),
             )
         if flows.password:
             return a2a_pb2.OAuthFlows(
                 password=a2a_pb2.PasswordOAuthFlow(
-                    refresh_url=flows.password.refreshUrl,
+                    refresh_url=flows.password.refresh_url,
                     scopes=dict(flows.password.scopes.items()),
-                    token_url=flows.password.tokenUrl,
+                    token_url=flows.password.token_url,
                 ),
             )
         raise ValueError('Unknown oauth flow definition')
@@ -415,8 +464,8 @@ class ToProto:
             description=skill.description,
             tags=skill.tags,
             examples=skill.examples,
-            input_modes=skill.inputModes,
-            output_modes=skill.outputModes,
+            input_modes=skill.input_modes,
+            output_modes=skill.output_modes,
         )
 
     @classmethod
@@ -436,34 +485,28 @@ class FromProto:
     @classmethod
     def message(cls, message: a2a_pb2.Message) -> types.Message:
         return types.Message(
-            messageId=message.message_id,
-            parts=[FromProto.part(p) for p in message.content],
-            contextId=message.context_id,
-            taskId=message.task_id,
-            role=FromProto.role(message.role),
-            metadata=FromProto.metadata(message.metadata),
+            message_id=message.message_id,
+            parts=[cls.part(p) for p in message.content],
+            context_id=message.context_id or None,
+            task_id=message.task_id or None,
+            role=cls.role(message.role),
+            metadata=cls.metadata(message.metadata),
         )
 
     @classmethod
     def metadata(cls, metadata: struct_pb2.Struct) -> dict[str, Any]:
-        return {
-            key: value.string_value
-            for key, value in metadata.fields.items()
-            if value.string_value
-        }
+        if not metadata.fields:
+            return {}
+        return json_format.MessageToDict(metadata)
 
     @classmethod
     def part(cls, part: a2a_pb2.Part) -> types.Part:
         if part.HasField('text'):
             return types.Part(root=types.TextPart(text=part.text))
         if part.HasField('file'):
-            return types.Part(
-                root=types.FilePart(file=FromProto.file(part.file))
-            )
+            return types.Part(root=types.FilePart(file=cls.file(part.file)))
         if part.HasField('data'):
-            return types.Part(
-                root=types.DataPart(data=FromProto.data(part.data))
-            )
+            return types.Part(root=types.DataPart(data=cls.data(part.data)))
         raise ValueError(f'Unsupported part type: {part}')
 
     @classmethod
@@ -475,25 +518,43 @@ class FromProto:
     def file(
         cls, file: a2a_pb2.FilePart
     ) -> types.FileWithUri | types.FileWithBytes:
+        common_args = {
+            'mime_type': file.mime_type or None,
+            'name': file.name or None,
+        }
         if file.HasField('file_with_uri'):
-            return types.FileWithUri(uri=file.file_with_uri)
-        return types.FileWithBytes(bytes=file.file_with_bytes.decode('utf-8'))
+            return types.FileWithUri(
+                uri=file.file_with_uri,
+                **common_args,
+            )
+        return types.FileWithBytes(
+            bytes=file.file_with_bytes.decode('utf-8'),
+            **common_args,
+        )
+
+    @classmethod
+    def task_or_message(
+        cls, event: a2a_pb2.SendMessageResponse
+    ) -> types.Task | types.Message:
+        if event.HasField('msg'):
+            return cls.message(event.msg)
+        return cls.task(event.task)
 
     @classmethod
     def task(cls, task: a2a_pb2.Task) -> types.Task:
         return types.Task(
             id=task.id,
-            contextId=task.context_id,
-            status=FromProto.task_status(task.status),
-            artifacts=[FromProto.artifact(a) for a in task.artifacts],
-            history=[FromProto.message(h) for h in task.history],
+            context_id=task.context_id,
+            status=cls.task_status(task.status),
+            artifacts=[cls.artifact(a) for a in task.artifacts],
+            history=[cls.message(h) for h in task.history],
         )
 
     @classmethod
     def task_status(cls, status: a2a_pb2.TaskStatus) -> types.TaskStatus:
         return types.TaskStatus(
-            state=FromProto.task_state(status.state),
-            message=FromProto.message(status.update),
+            state=cls.task_state(status.state),
+            message=cls.message(status.update),
         )
 
     @classmethod
@@ -511,17 +572,19 @@ class FromProto:
                 return types.TaskState.failed
             case a2a_pb2.TaskState.TASK_STATE_INPUT_REQUIRED:
                 return types.TaskState.input_required
+            case a2a_pb2.TaskState.TASK_STATE_AUTH_REQUIRED:
+                return types.TaskState.auth_required
             case _:
                 return types.TaskState.unknown
 
     @classmethod
     def artifact(cls, artifact: a2a_pb2.Artifact) -> types.Artifact:
         return types.Artifact(
-            artifactId=artifact.artifact_id,
+            artifact_id=artifact.artifact_id,
             description=artifact.description,
-            metadata=FromProto.metadata(artifact.metadata),
+            metadata=cls.metadata(artifact.metadata),
             name=artifact.name,
-            parts=[FromProto.part(p) for p in artifact.parts],
+            parts=[cls.part(p) for p in artifact.parts],
         )
 
     @classmethod
@@ -529,12 +592,12 @@ class FromProto:
         cls, event: a2a_pb2.TaskArtifactUpdateEvent
     ) -> types.TaskArtifactUpdateEvent:
         return types.TaskArtifactUpdateEvent(
-            taskId=event.task_id,
-            contextId=event.context_id,
-            artifact=FromProto.artifact(event.artifact),
-            metadata=FromProto.metadata(event.metadata),
+            task_id=event.task_id,
+            context_id=event.context_id,
+            artifact=cls.artifact(event.artifact),
+            metadata=cls.metadata(event.metadata),
             append=event.append,
-            lastChunk=event.last_chunk,
+            last_chunk=event.last_chunk,
         )
 
     @classmethod
@@ -542,10 +605,10 @@ class FromProto:
         cls, event: a2a_pb2.TaskStatusUpdateEvent
     ) -> types.TaskStatusUpdateEvent:
         return types.TaskStatusUpdateEvent(
-            taskId=event.task_id,
-            contextId=event.context_id,
-            status=FromProto.task_status(event.status),
-            metadata=FromProto.metadata(event.metadata),
+            task_id=event.task_id,
+            context_id=event.context_id,
+            status=cls.task_status(event.status),
+            metadata=cls.metadata(event.metadata),
             final=event.final,
         )
 
@@ -557,7 +620,7 @@ class FromProto:
             id=config.id,
             url=config.url,
             token=config.token,
-            authentication=FromProto.authentication_info(config.authentication)
+            authentication=cls.authentication_info(config.authentication)
             if config.HasField('authentication')
             else None,
         )
@@ -576,13 +639,13 @@ class FromProto:
         cls, config: a2a_pb2.SendMessageConfiguration
     ) -> types.MessageSendConfiguration:
         return types.MessageSendConfiguration(
-            acceptedOutputModes=list(config.accepted_output_modes),
-            pushNotificationConfig=FromProto.push_notification_config(
+            accepted_output_modes=list(config.accepted_output_modes),
+            push_notification_config=cls.push_notification_config(
                 config.push_notification
             )
             if config.HasField('push_notification')
             else None,
-            historyLength=config.history_length,
+            history_length=config.history_length,
             blocking=config.blocking,
         )
 
@@ -605,10 +668,8 @@ class FromProto:
             | a2a_pb2.GetTaskPushNotificationConfigRequest
         ),
     ) -> types.TaskIdParams:
-        # This is currently incomplete until the core sdk supports multiple
-        # configs for a single task.
         if isinstance(request, a2a_pb2.GetTaskPushNotificationConfigRequest):
-            m = re.match(_TASK_PUSH_CONFIG_NAME_MATCH, request.name)
+            m = _TASK_PUSH_CONFIG_NAME_MATCH.match(request.name)
             if not m:
                 raise ServerError(
                     error=types.InvalidParamsError(
@@ -616,7 +677,7 @@ class FromProto:
                     )
                 )
             return types.TaskIdParams(id=m.group(1))
-        m = re.match(_TASK_NAME_MATCH, request.name)
+        m = _TASK_NAME_MATCH.match(request.name)
         if not m:
             raise ServerError(
                 error=types.InvalidParamsError(
@@ -626,11 +687,11 @@ class FromProto:
         return types.TaskIdParams(id=m.group(1))
 
     @classmethod
-    def task_push_notification_config(
+    def task_push_notification_config_request(
         cls,
         request: a2a_pb2.CreateTaskPushNotificationConfigRequest,
     ) -> types.TaskPushNotificationConfig:
-        m = re.match(_TASK_NAME_MATCH, request.parent)
+        m = _TASK_NAME_MATCH.match(request.parent)
         if not m:
             raise ServerError(
                 error=types.InvalidParamsError(
@@ -638,10 +699,29 @@ class FromProto:
                 )
             )
         return types.TaskPushNotificationConfig(
-            pushNotificationConfig=cls.push_notification_config(
+            push_notification_config=cls.push_notification_config(
                 request.config.push_notification_config,
             ),
-            taskId=m.group(1),
+            task_id=m.group(1),
+        )
+
+    @classmethod
+    def task_push_notification_config(
+        cls,
+        config: a2a_pb2.TaskPushNotificationConfig,
+    ) -> types.TaskPushNotificationConfig:
+        m = _TASK_PUSH_CONFIG_NAME_MATCH.match(config.name)
+        if not m:
+            raise ServerError(
+                error=types.InvalidParamsError(
+                    message=f'Bad TaskPushNotificationConfig resource name {config.name}'
+                )
+            )
+        return types.TaskPushNotificationConfig(
+            push_notification_config=cls.push_notification_config(
+                config.push_notification_config,
+            ),
+            task_id=m.group(1),
         )
 
     @classmethod
@@ -651,18 +731,35 @@ class FromProto:
     ) -> types.AgentCard:
         return types.AgentCard(
             capabilities=cls.capabilities(card.capabilities),
-            defaultInputModes=list(card.default_input_modes),
-            defaultOutputModes=list(card.default_output_modes),
+            default_input_modes=list(card.default_input_modes),
+            default_output_modes=list(card.default_output_modes),
             description=card.description,
-            documentationUrl=card.documentation_url,
+            documentation_url=card.documentation_url,
             name=card.name,
             provider=cls.provider(card.provider),
             security=cls.security(list(card.security)),
-            securitySchemes=cls.security_schemes(dict(card.security_schemes)),
+            security_schemes=cls.security_schemes(dict(card.security_schemes)),
             skills=[cls.skill(x) for x in card.skills] if card.skills else [],
             url=card.url,
             version=card.version,
-            supportsAuthenticatedExtendedCard=card.supports_authenticated_extended_card,
+            supports_authenticated_extended_card=card.supports_authenticated_extended_card,
+            preferred_transport=card.preferred_transport,
+            protocol_version=card.protocol_version,
+            additional_interfaces=[
+                cls.agent_interface(x) for x in card.additional_interfaces
+            ]
+            if card.additional_interfaces
+            else None,
+        )
+
+    @classmethod
+    def agent_interface(
+        cls,
+        interface: a2a_pb2.AgentInterface,
+    ) -> types.AgentInterface:
+        return types.AgentInterface(
+            transport=interface.transport,
+            url=interface.url,
         )
 
     @classmethod
@@ -670,7 +767,7 @@ class FromProto:
         cls,
         request: a2a_pb2.GetTaskRequest,
     ) -> types.TaskQueryParams:
-        m = re.match(_TASK_NAME_MATCH, request.name)
+        m = _TASK_NAME_MATCH.match(request.name)
         if not m:
             raise ServerError(
                 error=types.InvalidParamsError(
@@ -678,7 +775,7 @@ class FromProto:
                 )
             )
         return types.TaskQueryParams(
-            historyLength=request.history_length
+            history_length=request.history_length
             if request.history_length
             else None,
             id=m.group(1),
@@ -691,7 +788,22 @@ class FromProto:
     ) -> types.AgentCapabilities:
         return types.AgentCapabilities(
             streaming=capabilities.streaming,
-            pushNotifications=capabilities.push_notifications,
+            push_notifications=capabilities.push_notifications,
+            extensions=[
+                cls.agent_extension(x) for x in capabilities.extensions
+            ],
+        )
+
+    @classmethod
+    def agent_extension(
+        cls,
+        extension: a2a_pb2.AgentExtension,
+    ) -> types.AgentExtension:
+        return types.AgentExtension(
+            uri=extension.uri,
+            description=extension.description,
+            params=json_format.MessageToDict(extension.params),
+            required=extension.required,
         )
 
     @classmethod
@@ -701,10 +813,9 @@ class FromProto:
     ) -> list[dict[str, list[str]]] | None:
         if not security:
             return None
-        rval: list[dict[str, list[str]]] = []
-        for s in security:
-            rval.append({k: list(v.list) for (k, v) in s.schemes.items()})
-        return rval
+        return [
+            {k: list(v.list) for (k, v) in s.schemes.items()} for s in security
+        ]
 
     @classmethod
     def provider(
@@ -741,7 +852,7 @@ class FromProto:
                 root=types.HTTPAuthSecurityScheme(
                     description=scheme.http_auth_security_scheme.description,
                     scheme=scheme.http_auth_security_scheme.scheme,
-                    bearerFormat=scheme.http_auth_security_scheme.bearer_format,
+                    bearer_format=scheme.http_auth_security_scheme.bearer_format,
                 )
             )
         if scheme.HasField('oauth2_security_scheme'):
@@ -751,10 +862,16 @@ class FromProto:
                     flows=cls.oauth2_flows(scheme.oauth2_security_scheme.flows),
                 )
             )
+        if scheme.HasField('mtls_security_scheme'):
+            return types.SecurityScheme(
+                root=types.MutualTLSSecurityScheme(
+                    description=scheme.mtls_security_scheme.description,
+                )
+            )
         return types.SecurityScheme(
             root=types.OpenIdConnectSecurityScheme(
                 description=scheme.open_id_connect_security_scheme.description,
-                openIdConnectUrl=scheme.open_id_connect_security_scheme.open_id_connect_url,
+                open_id_connect_url=scheme.open_id_connect_security_scheme.open_id_connect_url,
             )
         )
 
@@ -762,37 +879,57 @@ class FromProto:
     def oauth2_flows(cls, flows: a2a_pb2.OAuthFlows) -> types.OAuthFlows:
         if flows.HasField('authorization_code'):
             return types.OAuthFlows(
-                authorizationCode=types.AuthorizationCodeOAuthFlow(
-                    authorizationUrl=flows.authorization_code.authorization_url,
-                    refreshUrl=flows.authorization_code.refresh_url,
+                authorization_code=types.AuthorizationCodeOAuthFlow(
+                    authorization_url=flows.authorization_code.authorization_url,
+                    refresh_url=flows.authorization_code.refresh_url,
                     scopes=dict(flows.authorization_code.scopes.items()),
-                    tokenUrl=flows.authorization_code.token_url,
+                    token_url=flows.authorization_code.token_url,
                 ),
             )
         if flows.HasField('client_credentials'):
             return types.OAuthFlows(
-                clientCredentials=types.ClientCredentialsOAuthFlow(
-                    refreshUrl=flows.client_credentials.refresh_url,
+                client_credentials=types.ClientCredentialsOAuthFlow(
+                    refresh_url=flows.client_credentials.refresh_url,
                     scopes=dict(flows.client_credentials.scopes.items()),
-                    tokenUrl=flows.client_credentials.token_url,
+                    token_url=flows.client_credentials.token_url,
                 ),
             )
         if flows.HasField('implicit'):
             return types.OAuthFlows(
                 implicit=types.ImplicitOAuthFlow(
-                    authorizationUrl=flows.implicit.authorization_url,
-                    refreshUrl=flows.implicit.refresh_url,
+                    authorization_url=flows.implicit.authorization_url,
+                    refresh_url=flows.implicit.refresh_url,
                     scopes=dict(flows.implicit.scopes.items()),
                 ),
             )
         return types.OAuthFlows(
             password=types.PasswordOAuthFlow(
-                refreshUrl=flows.password.refresh_url,
+                refresh_url=flows.password.refresh_url,
                 scopes=dict(flows.password.scopes.items()),
-                tokenUrl=flows.password.token_url,
+                token_url=flows.password.token_url,
             ),
         )
 
+    @classmethod
+    def stream_response(
+        cls,
+        response: a2a_pb2.StreamResponse,
+    ) -> (
+        types.Message
+        | types.Task
+        | types.TaskStatusUpdateEvent
+        | types.TaskArtifactUpdateEvent
+    ):
+        if response.HasField('msg'):
+            return cls.message(response.msg)
+        if response.HasField('task'):
+            return cls.task(response.task)
+        if response.HasField('status_update'):
+            return cls.task_status_update_event(response.status_update)
+        if response.HasField('artifact_update'):
+            return cls.task_artifact_update_event(response.artifact_update)
+        raise ValueError('Unsupported StreamResponse type')
+
     @classmethod
     def skill(cls, skill: a2a_pb2.AgentSkill) -> types.AgentSkill:
         return types.AgentSkill(
@@ -801,8 +938,8 @@ class FromProto:
             description=skill.description,
             tags=list(skill.tags),
             examples=list(skill.examples),
-            inputModes=list(skill.input_modes),
-            outputModes=list(skill.output_modes),
+            input_modes=list(skill.input_modes),
+            output_modes=list(skill.output_modes),
         )
 
     @classmethod
